#' Create, read, update, delete, or report status of an edge relationship
#' From a graph object of class 'gv_graph', query an edge in the graph (defined by a pair of node IDs extant in the graph) and perform operations on the relationship for that edge.
#' @param graph a graph object of class 'gv_graph' that is created using 'create_graph'.
#' @param from a node ID from which the edge to be queried is outgoing.
#' @param to a node ID to which the edge to be queried is incoming.
#' @param action the type of operation to perform post-query. To remove a relationship from an edge, use either 'delete', 'remove', or 'drop'. To add a relationship to an edge with no set relationship, use 'add' or 'create'. To update an edge relationship, use 'update'. To return the value of an edge relationship, use 'read'. To determine whether there is a set relationship, use 'check'.
#' @param value a string denoting the relationship, to be supplied when either adding or updating an edge relationship.
#' @return a graph object of class 'gv_graph'.
#' @export edge_relationship

edge_relationship <- function(graph,
                              from,
                              to,
                              action = "read",
                              value = NULL){

  # Determine if edge is present within the graph
  edge_is_in_graph <- edge_present(graph = graph, from = from, to = to)

  # Stop function if edge is not present within the graph
  if (edge_is_in_graph == FALSE){

    stop("The specified edge is not present in the graph.")
  }

  if (edge_is_in_graph == TRUE){

    edge_row <- which(graph$edges_df$edge_from == from & graph$edges_df$edge_to == to)

    relationship_set <- ifelse(graph$edges_df$relationship[edge_row] == "",
                               FALSE, TRUE)

    # Remove relationship if a relationship is set
    if (action %in% c("delete", "remove", "drop")){

      if (relationship_set == FALSE){

        return(graph)
      }

      if (relationship_set == TRUE){

        graph$edges_df$relationship[edge_row] <- ""

        return(graph)
      }
    }

    # Add a relationship to an edge with no set relationship
    if (action %in% c("add", "create")){

      if (relationship_set == TRUE){

        return(graph)
      }

      if (relationship_set == FALSE & !is.null(value)){

        graph$edges_df$relationship[edge_row] <- value

        return(graph)
      }
    }

    # Update an existing relationship for an edge
    if (action == "update"){

      if (relationship_set == FALSE){

        return(graph)
      }

      if (relationship_set == TRUE & !is.null(value)){

        graph$edges_df$relationship[edge_row] <- value

        return(graph)
      }
    }

    # Return the value of an existing relationship for an edge
    if (action == "read"){

      if (relationship_set == FALSE){

        return(NA)
      }

      if (relationship_set == TRUE){

        relationship_value <- graph$edges_df$relationship[edge_row]

        return(relationship_value)
      }
    }

    # Determine whether a relationship has been set
    if (action == "check"){

      if (relationship_set == FALSE){

        return(FALSE)
      }

      if (relationship_set == TRUE){

        return(TRUE)
      }
    }
  }
}
